package cn.ibizlab.api.rest;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;

import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;

import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.access.prepost.PreAuthorize;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import cn.ibizlab.api.dto.*;
import cn.ibizlab.api.mapping.*;
import cn.ibizlab.core.workflow.domain.WFSystem;
import cn.ibizlab.core.workflow.service.IWFSystemService;
import cn.ibizlab.core.workflow.filter.WFSystemSearchContext;




@Slf4j
@Api(tags = {"WFSystem" })
@RestController("api-wfsystem")
@RequestMapping("")
public class WFSystemResource {

    @Autowired
    private IWFSystemService wfsystemService;

    @Autowired
    @Lazy
    public WFSystemMapping wfsystemMapping;

    public WFSystemDTO permissionDTO=new WFSystemDTO();

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Remove-all')")
    @ApiOperation(value = "Remove", tags = {"WFSystem" },  notes = "Remove")
	@RequestMapping(method = RequestMethod.DELETE, value = "/wfsystems/{wfsystem_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("wfsystem_id") String wfsystem_id) {
         return ResponseEntity.status(HttpStatus.OK).body(wfsystemService.remove(wfsystem_id));
    }

    @PreAuthorize("hasPermission('Remove',{'Sql',this.humanMapping,this.permissionDTO,#ids})")
    @ApiOperation(value = "RemoveBatch", tags = {"WFSystem" },  notes = "RemoveBatch")
	@RequestMapping(method = RequestMethod.DELETE, value = "/wfsystems/batch")
    public ResponseEntity<Boolean> removeBatch(@RequestBody List<String> ids) {
        wfsystemService.removeBatch(ids);
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Update-all')")
    @ApiOperation(value = "Update", tags = {"WFSystem" },  notes = "Update")
	@RequestMapping(method = RequestMethod.PUT, value = "/wfsystems/{wfsystem_id}")
    @Transactional
    public ResponseEntity<WFSystemDTO> update(@PathVariable("wfsystem_id") String wfsystem_id, @RequestBody WFSystemDTO wfsystemdto) {
		WFSystem domain = wfsystemMapping.toDomain(wfsystemdto);
        domain.setPssystemid(wfsystem_id);
		wfsystemService.update(domain);
		WFSystemDTO dto = wfsystemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission('Update',{'Sql',this.humanMapping,#humandtos})")
    @ApiOperation(value = "UpdateBatch", tags = {"WFSystem" },  notes = "UpdateBatch")
	@RequestMapping(method = RequestMethod.PUT, value = "/wfsystems/batch")
    public ResponseEntity<Boolean> updateBatch(@RequestBody List<WFSystemDTO> wfsystemdtos) {
        wfsystemService.updateBatch(wfsystemMapping.toDomain(wfsystemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-GetDraft-all')")
    @ApiOperation(value = "GetDraft", tags = {"WFSystem" },  notes = "GetDraft")
	@RequestMapping(method = RequestMethod.GET, value = "/wfsystems/getdraft")
    public ResponseEntity<WFSystemDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(wfsystemMapping.toDto(wfsystemService.getDraft(new WFSystem())));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Save-all')")
    @ApiOperation(value = "Save", tags = {"WFSystem" },  notes = "Save")
	@RequestMapping(method = RequestMethod.POST, value = "/wfsystems/save")
    public ResponseEntity<Boolean> save(@RequestBody WFSystemDTO wfsystemdto) {
        return ResponseEntity.status(HttpStatus.OK).body(wfsystemService.save(wfsystemMapping.toDomain(wfsystemdto)));
    }

    @PreAuthorize("hasPermission('Save',{'Sql',this.humanMapping,#humandtos})")
    @ApiOperation(value = "SaveBatch", tags = {"WFSystem" },  notes = "SaveBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/wfsystems/savebatch")
    public ResponseEntity<Boolean> saveBatch(@RequestBody List<WFSystemDTO> wfsystemdtos) {
        wfsystemService.saveBatch(wfsystemMapping.toDomain(wfsystemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Create-all')")
    @ApiOperation(value = "Create", tags = {"WFSystem" },  notes = "Create")
	@RequestMapping(method = RequestMethod.POST, value = "/wfsystems")
    @Transactional
    public ResponseEntity<WFSystemDTO> create(@RequestBody WFSystemDTO wfsystemdto) {
        WFSystem domain = wfsystemMapping.toDomain(wfsystemdto);
		wfsystemService.create(domain);
        WFSystemDTO dto = wfsystemMapping.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasPermission('Create',{'Sql',this.humanMapping,#humandtos})")
    @ApiOperation(value = "createBatch", tags = {"WFSystem" },  notes = "createBatch")
	@RequestMapping(method = RequestMethod.POST, value = "/wfsystems/batch")
    public ResponseEntity<Boolean> createBatch(@RequestBody List<WFSystemDTO> wfsystemdtos) {
        wfsystemService.createBatch(wfsystemMapping.toDomain(wfsystemdtos));
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-CheckKey-all')")
    @ApiOperation(value = "CheckKey", tags = {"WFSystem" },  notes = "CheckKey")
	@RequestMapping(method = RequestMethod.POST, value = "/wfsystems/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody WFSystemDTO wfsystemdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(wfsystemService.checkKey(wfsystemMapping.toDomain(wfsystemdto)));
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Get-all')")
    @ApiOperation(value = "Get", tags = {"WFSystem" },  notes = "Get")
	@RequestMapping(method = RequestMethod.GET, value = "/wfsystems/{wfsystem_id}")
    public ResponseEntity<WFSystemDTO> get(@PathVariable("wfsystem_id") String wfsystem_id) {
        WFSystem domain = wfsystemService.get(wfsystem_id);
        WFSystemDTO dto = wfsystemMapping.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Default-all')")
	@ApiOperation(value = "fetchDEFAULT", tags = {"WFSystem" } ,notes = "fetchDEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/wfsystems/fetchdefault")
	public ResponseEntity<List<WFSystemDTO>> fetchDefault(WFSystemSearchContext context) {
        Page<WFSystem> domains = wfsystemService.searchDefault(context) ;
        List<WFSystemDTO> list = wfsystemMapping.toDto(domains.getContent());
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}

    @PreAuthorize("hasAnyAuthority('ROLE_SUPERADMIN','ibzwf-WFSystem-Default-all')")
	@ApiOperation(value = "searchDEFAULT", tags = {"WFSystem" } ,notes = "searchDEFAULT")
    @RequestMapping(method= RequestMethod.POST , value="/wfsystems/searchdefault")
	public ResponseEntity<Page<WFSystemDTO>> searchDefault(@RequestBody WFSystemSearchContext context) {
        Page<WFSystem> domains = wfsystemService.searchDefault(context) ;
	    return ResponseEntity.status(HttpStatus.OK)
                .body(new PageImpl(wfsystemMapping.toDto(domains.getContent()), context.getPageable(), domains.getTotalElements()));
	}
}
